from selenium.webdriver.firefox.webdriver import WebDriver

from ..action import callback
from ..loader import loader


def click_page_button(driver: WebDriver, selector: str):
    loader.get_page_source_until_all_selectors(driver=driver, selector=selector)
    next_page = driver.find_element_by_css_selector(selector)
    callback.function_call_random_delay(next_page.click)



import csv
import shutil
import logger


def append_array_to_csv_file(csv_file_path: str, csv_array: list):
    with open(csv_file_path, 'a') as csv_file:
        writer = csv.writer(csv_file)
        for csv_row in csv_array:
            writer.writerow(csv_row.__dict__.values())
            logger.debug('Inserted in to csv file {} row: {}'
                         .format(csv_file.name, csv_row.__dict__.values()))


def clean_csv_file(csv_blank_file_path: str, csv_file_path: str):
    shutil.copyfile(csv_blank_file_path, csv_file_path)
    logger.debug('Cleaned file: {}'.format(csv_blank_file_path))



from bs4 import BeautifulSoup
from requests import Session


def get_page_soup(session: Session, url: str):
    page = session.get(url)
    return BeautifulSoup(page.content, features="lxml")


def get_soup_by_content(page_content: str):
    return BeautifulSoup(page_content, features="lxml")


def attribute_value_for_all_elements(soup: BeautifulSoup, selector: str, attribute: str):
    elements = soup.select(selector)
    return [e.get(attribute) for e in elements if e.get(attribute, '') != '']


def attribute_value_element(soup: BeautifulSoup, selector: str, attribute: str):
    elements = soup.select(selector)
    img_urls = [e.get(attribute) for e in elements if e.get(attribute, '') != '']
    return img_urls[0] if len(img_urls) > 0 else ''


def href_url_index_0(soup: BeautifulSoup, selector: str):
    elements = soup.select(selector + ' a')
    return [a['href'] for a in elements][0]


def all_href_urls(soup: BeautifulSoup, selector: str):
    elements = soup.select(selector + ' a')
    return [a['href'] for a in elements if a.get('href', '') != '']


def images_src(soup: BeautifulSoup, selector: str):
    images = soup.select(selector + ' img')
    return [img['src'] for img in images]


def image_src(soup: BeautifulSoup, selector: str):
    img = soup.select(selector + ' img')[0]
    return img['src'] if img.get('src', '') != '' else ''


def tag_text(soup: BeautifulSoup, selector: str):
    return soup.select(selector)[0].text.replace('\r', '').replace('\n', '').replace('\t', '').strip() if len(
        soup.select(selector)) > 0 else ''


def tags_text(soup: BeautifulSoup, selector: str):
    return list(
        map(lambda x: str(x.text).replace('\r', '').replace('\n', '').replace('\t', '').strip(), soup.select(selector)))


def inner_html(soup: BeautifulSoup, selector: str):
    return soup.select(selector)


def inner_html_str(soup: BeautifulSoup, selector: str):
    return [str(html).replace('\r', '').replace('\n', '').replace('\t', '') for html in soup.select(selector)]


def inner_html_str_index_0(soup: BeautifulSoup, selector: str):
    return str(soup.select(selector)[0]).replace('\r', '').replace('\n', '').replace('\t', '') if len(
        soup.select(selector)) > 0 else ''


def inner_html_str_at_index(soup: BeautifulSoup, selector: str, index: int):
    return str(soup.select(selector)[index]).replace('\r', '').replace('\n', '').replace('\t', '')



import time
from random import randint

from ..loader import loader


def function_call_delay(function, delay: int = 1, **kwargs):
    time.sleep(delay)
    return function(**kwargs)


def function_call_random_delay(function, delay: int = 5, **kwargs):
    time.sleep(randint(2, delay))
    return function(**kwargs)


def function_call_page_loading_delay_by_tag(function, tag: str = 'img', delay: int = 1, **kwargs):
    try:
        return function(**kwargs)
    finally:
        loader.get_page_source_until_all_tags_with_delay(driver=kwargs.get('driver'), tag=tag, delay=delay)


def function_call_page_loading_delay_by_name(function, name: str, delay: int = 1, **kwargs):
    try:
        return function(**kwargs)
    finally:
        loader.get_page_source_until_all_names_with_delay(driver=kwargs.get('driver'), name=name, delay=delay)


def function_call_page_loading_delay_by_selector(function, selector: str, delay: int = 1, **kwargs):
    try:
        return function(**kwargs)
    finally:
        loader.get_page_source_until_all_selectors_with_delay(driver=kwargs.get('driver'), selector=selector,
                                                              delay=delay)



import time

from selenium.webdriver.common.by import By
from selenium.webdriver.firefox.webdriver import WebDriver
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.wait import WebDriverWait

from ..collector import extractor


def get_page_source_until_all_tags(driver: WebDriver, tag: str, timeout: int = 10):
    WebDriverWait(driver, timeout).until(EC.presence_of_all_elements_located((By.TAG_NAME, tag)))
    return driver.page_source


def get_page_source_until_all_tags_with_delay(driver: WebDriver, tag: str, timeout: int = 10, delay: int = 1):
    WebDriverWait(driver, timeout).until(EC.presence_of_all_elements_located((By.TAG_NAME, tag)))
    time.sleep(delay)
    return driver.page_source


def get_page_source_until_all_names(driver: WebDriver, name: str, timeout: int = 10):
    WebDriverWait(driver, timeout).until(EC.presence_of_all_elements_located((By.NAME, name)))
    return driver.page_source


def get_page_source_until_all_names_with_delay(driver: WebDriver, name: str, timeout: int = 10, delay: int = 1):
    WebDriverWait(driver, timeout).until(EC.presence_of_all_elements_located((By.NAME, name)))
    time.sleep(delay)
    return driver.page_source


def get_page_source_until_all_selectors(driver: WebDriver, selector: str, timeout: int = 10):
    WebDriverWait(driver, timeout).until(EC.presence_of_all_elements_located((By.CSS_SELECTOR, selector)))
    return driver.page_source


def get_page_source_until_all_selectors_with_delay(driver: WebDriver, selector: str, timeout: int = 10, delay: int = 1):
    WebDriverWait(driver, timeout).until(EC.presence_of_all_elements_located((By.CSS_SELECTOR, selector)))
    time.sleep(delay)
    return driver.page_source


def load_page_soup_by_all_tags_with_delay(driver: WebDriver, tag: str):
    page_content = get_page_source_until_all_tags_with_delay(driver=driver, tag=tag)
    return extractor.get_soup_by_content(page_content)


def load_page_soup_by_all_selectors_with_delay(driver: WebDriver, selector: str):
    page_content = get_page_source_until_all_selectors_with_delay(driver=driver, selector=selector)
    return extractor.get_soup_by_content(page_content)


def load_page_soup_by_all_names_with_delay(driver: WebDriver, name: str):
    page_content = get_page_source_until_all_names_with_delay(driver=driver, name=name)
    return extractor.get_soup_by_content(page_content)



from selenium.common.exceptions import NoSuchElementException
from selenium.webdriver.firefox.webdriver import WebDriver


def element_exists_by_id(id: str, driver: WebDriver):
    try:
        driver.find_element_by_id(id)
    except NoSuchElementException:
        return False
    return True


def element_exists_by_css_selector(css_selector: str, driver: WebDriver):
    try:
        driver.find_elements_by_css_selector(css_selector)
    except NoSuchElementException:
        return False
    return True


def element_exists_by_name(name: str, driver: WebDriver):
    try:
        driver.find_element_by_name(name)
    except NoSuchElementException:
        return False
    return True
